'From Squeak3.2gamma of 15 January 2002 [latest update: #4743] on 10 March 2002 at 12:24:22 am'!
Magnitude subclass: #MagmaCalendarTime
	instanceVariableNames: 'totalMilliseconds '
	classVariableNames: 'BaseCalendarDate FirstRecordedMillisecondClock FirstTimestampMillisecond '
	poolDictionaries: ''
	category: 'Magma-SUnit tests'!

!MagmaCalendarTime commentStamp: '<historical>' prior: 0!
I represent a calendar time; either a duration represented in units of a Gregorian calendar (days, hours, minutes, seconds, etc.) or a point on the timeline of the Gregorian calendar.

I support full arithmetic with other CalendarTimes and Integers as it makes sense.  For example, dividing 30 minutes by 3 makes sense, but not May 19th, 1969.

I have fairly efficient storage, a single integer of the total number of milliseconds that have elapsed since my base time, 1/1/1901 @ 12:00am.
!

MagmaCalendarTime class
	instanceVariableNames: ''!

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/7/2002 22:48'!
* anInteger

	^self class fromMilliseconds: self totalMilliseconds * anInteger
! !

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/6/2002 23:30'!
+ aMagmaCalendarTimeOrInteger

	^aMagmaCalendarTimeOrInteger maAddTimeStamp: self
! !

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/7/2002 22:49'!
- aMagmaCalendarTime

	^self class fromMilliseconds: self totalMilliseconds - aMagmaCalendarTime totalMilliseconds
! !

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/7/2002 22:46'!
// anIntegerOrCalendarTime

	^anIntegerOrCalendarTime maDivideIntoCalendarTime: self
! !

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/7/2002 22:48'!
\\ anIntegerOrCalendarTime

	^anIntegerOrCalendarTime maModuloCalendarTime: self
! !

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/7/2002 22:50'!
roundTo: aMagmaCalendarTime
	"aMagmaCalendarTime should be a duration.
	i.e., if the receiver is 5 minutes, 37 seconds, and aMagmaCalendarTime
	is 2 minutes, then return 6 minutes."

	^self class fromMilliseconds: (self totalMilliseconds roundTo: aMagmaCalendarTime totalMilliseconds)
! !

!MagmaCalendarTime methodsFor: 'operators' stamp: 'cmm 3/7/2002 22:50'!
truncateTo: aMagmaCalendarTime
	"aMagmaCalendarTime should be a duration.
	i.e., if the receiver is 5 minutes, 37 seconds, and duration
	is 2 minutes, then return 4 minutes."

	^self class
		fromMilliseconds: (self totalMilliseconds truncateTo: aMagmaCalendarTime totalMilliseconds)
! !

!MagmaCalendarTime methodsFor: 'comparing' stamp: 'cmm 3/6/2002 23:29'!
< aMagmaCalendarTime

	^self totalMilliseconds < aMagmaCalendarTime totalMilliseconds
! !

!MagmaCalendarTime methodsFor: 'comparing' stamp: 'cmm 3/6/2002 21:48'!
= aMagmaCalendarTime

	aMagmaCalendarTime class = MagmaCalendarTime ifFalse: [ ^false ].
	^self totalMilliseconds = aMagmaCalendarTime totalMilliseconds
! !

!MagmaCalendarTime methodsFor: 'converting'!
to: endTimeStamp by: stepDuration

	^MaTimeInterval
		from: self
		to: endTimeStamp
		by: stepDuration
! !

!MagmaCalendarTime methodsFor: 'converting'!
to: endTimeStamp by: durationIncrement do: aBlock
	"Evaluate aBlock for each MaTimeStamp element in the interval.
	Return stop."

	(self
		to: endTimeStamp
		by: durationIncrement) do: [ :each | aBlock value: each ].	
	^endTimeStamp
! !

!MagmaCalendarTime methodsFor: 'accessing' stamp: 'cmm 3/7/2002 22:57'!
date

	^Date fromDays: self totalDays
! !

!MagmaCalendarTime methodsFor: 'accessing'!
hash

	^self totalMilliseconds hash
! !

!MagmaCalendarTime methodsFor: 'accessing'!
milliseconds

	^self totalMilliseconds \\ 1000
! !

!MagmaCalendarTime methodsFor: 'accessing' stamp: 'cmm 3/7/2002 22:57'!
time

	^Time fromSeconds: self totalSeconds
! !

!MagmaCalendarTime methodsFor: 'accessing' stamp: 'cmm 3/6/2002 22:14'!
totalDays

	^self totalSeconds // MagmaCalendarTime secondsPerDay
! !

!MagmaCalendarTime methodsFor: 'accessing'!
totalMilliseconds

	^totalMilliseconds! !

!MagmaCalendarTime methodsFor: 'accessing'!
totalMilliseconds: anInteger

	totalMilliseconds _ anInteger
! !

!MagmaCalendarTime methodsFor: 'accessing'!
totalSeconds

	^self totalMilliseconds // 1000
! !

!MagmaCalendarTime methodsFor: 'building' stamp: 'cmm 3/7/2002 23:00'!
date: aDate time: aTime

	^self
		date: aDate
		time: aTime
		milliseconds: 0
! !

!MagmaCalendarTime methodsFor: 'building' stamp: 'cmm 3/7/2002 22:59'!
date: aDate time: aTime milliseconds: anInteger

	self totalMilliseconds: (aDate asSeconds + aTime asSeconds) * 1000 + anInteger
! !

!MagmaCalendarTime methodsFor: 'building'!
milliseconds: anInteger

	totalMilliseconds _ (totalMilliseconds truncateTo: 1000) + anInteger
! !

!MagmaCalendarTime methodsFor: 'building'!
totalSeconds: anInteger

	self totalMilliseconds: anInteger * 1000
! !

!MagmaCalendarTime methodsFor: 'private printing' stamp: 'cmm 3/7/2002 23:01'!
isDuration
	"Returns true if the receiver most likely is representing a duration
	instead of a point in time.  Use with caution, as you may really want
	to be measuring a duration.  However, if this were a point in time, it
	would be before 1951..."

	^totalMilliseconds < 1576800000000
! !

!MagmaCalendarTime methodsFor: 'private printing' stamp: 'cmm 3/6/2002 23:46'!
printDurationOn: aStream

	(self totalSeconds >= MagmaCalendarTime secondsPerDay)
		ifTrue:
			[ aStream
				maPrint: 
					(MaTextMessage new
						appendCount: (self date subtractDate: BaseCalendarDate)
						of: 'day';
					text);
				nextPutAll: ', ' ].
	aStream nextPutAll: self time print24
! !

!MagmaCalendarTime methodsFor: 'private printing'!
printDurationWithMillisecondsOn: aStream

	self
		printDurationOn: aStream;
		printMillisecondsOn: aStream
! !

!MagmaCalendarTime methodsFor: 'private printing'!
printMillisecondsOn: aStream

	aStream
		nextPut: $.;
		nextPutAll: (
			self milliseconds printString
				abtPadWith: $0
				upToLength: 3
				onRight: false)
! !

!MagmaCalendarTime methodsFor: 'private private - dispatch' stamp: 'cmm 3/7/2002 22:49'!
maAddTimeStamp: aMagmaCalendarTime

	^self class fromMilliseconds: self totalMilliseconds + aMagmaCalendarTime totalMilliseconds
! !

!MagmaCalendarTime methodsFor: 'private private - dispatch' stamp: 'cmm 3/7/2002 22:49'!
maDivideIntoTimeStamp: aMagmaCalendarTime

	^self class fromMilliseconds: aMagmaCalendarTime totalMilliseconds // self totalMilliseconds
! !

!MagmaCalendarTime methodsFor: 'private private - dispatch' stamp: 'cmm 3/7/2002 22:50'!
maModuloTimeStamp: aMagmaCalendarTime

	^self class fromMilliseconds: aMagmaCalendarTime totalMilliseconds \\ self totalMilliseconds
! !

!MagmaCalendarTime methodsFor: 'printing' stamp: 'cmm 3/7/2002 22:57'!
printAbbreviatedOn: aStream

	self isDuration
		ifTrue:
			[ self printDurationOn: aStream ]
		ifFalse:
			[ aStream
				maPrint: self date;
				space;
				maPrint: self time ]
! !

!MagmaCalendarTime methodsFor: 'printing'!
printOn: aStream

	super printOn: aStream.
	aStream space.
	self printAbbreviatedOn: aStream
! !

!MagmaCalendarTime methodsFor: 'creation' stamp: 'cmm 3/7/2002 23:01'!
to: aMagmaCalendarTime

	^self
		to: aMagmaCalendarTime
		by: 1 second
! !


!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/7/2002 22:40'!
date: aDate time: aTime

	^self fromSeconds: aDate asSeconds + aTime asSeconds
! !

!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/7/2002 22:50'!
date: aDate time: aTime milliseconds: anInteger

	^self fromMilliseconds: (aDate asSeconds + aTime asSeconds) * 1000 + anInteger
! !

!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/7/2002 22:51'!
epoch
	"Return a new instance of the receiver with its time set to 0.
	This is useful for timing operations."

	^self fromMilliseconds: 0
! !

!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/7/2002 22:44'!
fromMilliseconds: anInteger

	^self new totalMilliseconds: anInteger
! !

!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/7/2002 22:39'!
fromSeconds: anInteger

	"Returns a new instance of the receiver based on anInteger seconds."

	^self new totalSeconds: anInteger
! !

!MagmaCalendarTime class methodsFor: 'creation'!
fromString: aString
	"Correct form is 'yyyy/mm/dd hh:mm:ss:nnn' or 'hh:mm:ss.nnn'
	if you intend to use the new instance as a 'duration'."

	^self
		fromString: aString
		ifInvalid: [ self error: aString, ' is not formatted properly for the default, simple parser.' ]
! !

!MagmaCalendarTime class methodsFor: 'creation'!
fromString: aString ifInvalid: aBlock
	"Correct form is 'yyyy/mm/dd hh:mm:ss:nnn' or 'hh:mm:ss.nnn'
	if you intend to use the new instance as a 'duration'."

	^aString size = 12
		ifTrue:
			[ self
				fromDurationString: aString
				ifInvalid: aBlock ]
		ifFalse:
			[ self
				fromTimeStampString: aString
				ifInvalid: aBlock ]
! !

!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/7/2002 22:51'!
now
	"answer a new timestamp for the current date and time."

	^self fromMilliseconds:
		(Time millisecondClockValue - FirstRecordedMillisecondClock + FirstTimestampMillisecond)
! !

!MagmaCalendarTime class methodsFor: 'creation' stamp: 'cmm 3/6/2002 22:13'!
secondsPerDay

	^86400
! !

!MagmaCalendarTime class methodsFor: 'private creation' stamp: 'cmm 3/7/2002 22:51'!
durationFromStream: aReadStream plusMilliseconds: anInteger ifInvalid: aBlock
	"Correct form is hh:mm:ss.nnn."

	| hours minutes seconds subseconds |

	(hours _ (aReadStream upTo: $:) asNumber) isNil ifTrue: [ ^aBlock value ].
	(minutes _ (aReadStream upTo: $:) asNumber) isNil ifTrue: [ ^aBlock value ].
	(seconds _ (aReadStream upTo: $.) asNumber) isNil ifTrue: [ ^aBlock value ].
	(subseconds _ aReadStream upToEnd asNumber) isNil ifTrue: [ ^aBlock value ].
	subseconds > 999 ifTrue: [ ^aBlock value ].
	^self fromMilliseconds:
		anInteger + (hours * 3600 * 1000) + (minutes * 60 * 1000) + (seconds * 1000) + subseconds
! !

!MagmaCalendarTime class methodsFor: 'private creation'!
fromDurationString: aString ifInvalid: aBlock
	"Correct form is hh:mm:ss.nnn."

	| stream |

	stream _ ReadStream on: aString trimBlanks.
	^self
		durationFromStream: stream
		plusMilliseconds: 0
		ifInvalid: aBlock
! !

!MagmaCalendarTime class methodsFor: 'private creation' stamp: 'cmm 3/7/2002 22:31'!
fromTimeStampString: aString ifInvalid: aBlock
	"Correct form is yyyy/mm/dd hh:mm:ss.mmm"

	| stream year month day |

	stream _ ReadStream on: aString.
	(year _ (stream upTo: $/) asNumber) isNil ifTrue: [ ^aBlock value ].
	(month _ (stream upTo: $/) asNumber) isNil ifTrue: [ ^aBlock value ].
	(day _ (stream upTo: $ ) asNumber) isNil ifTrue: [ ^aBlock value ].
	^self
		durationFromStream: stream
		plusMilliseconds:
			(Date
				newDay: day
				month: month
				year: year)
					asSeconds * 1000
		ifInvalid: aBlock
! !

!MagmaCalendarTime class methodsFor: 'private initializing' stamp: 'cmm 3/6/2002 23:44'!
initialize

	FirstRecordedMillisecondClock _ Time millisecondClockValue.
	FirstTimestampMillisecond _ (Date today asSeconds + Time now asSeconds) * 1000.
	BaseCalendarDate _ Date newDay: 1 month: 1 year: 1901
! !

MagmaCalendarTime initialize!
MagmaCalendarTime class removeSelector: #nextEpoch!
MagmaCalendarTime class removeSelector: #totalMilliseconds:!
MagmaCalendarTime class removeSelector: #totalSeconds:!
MagmaCalendarTime removeSelector: #asDate!
MagmaCalendarTime removeSelector: #asTime!
